@@ -5,6 +5,7 @@
 //!
 //! [Ruff]: https://github.com/astral-sh/ruff
 
+pub use noqa::generate_noqa_edits;
 #[cfg(feature = "clap")]
 pub use registry::clap_completion::RuleParser;
 #[cfg(feature = "clap")]
@@ -24,7 +25,6 @@ mod docstrings;
 mod fix;
 pub mod fs;
 mod importer;
-mod lex;
 pub mod line_width;
 pub mod linter;
 pub mod logging;
