@@ -301,7 +301,7 @@ pub(crate) fn is_same_expr<'a>(a: &'a Expr, b: &'a Expr) -> Option<&'a str> {
 /// If `call` is an `isinstance()` call, return its target.
 fn isinstance_target<'a>(call: &'a Expr, semantic: &'a SemanticModel) -> Option<&'a Expr> {
     // Verify that this is an `isinstance` call.
-    let Expr::Call(ast::ExprCall {
+    let ast::ExprCall {
         func,
         arguments:
             Arguments {
@@ -310,23 +310,14 @@ fn isinstance_target<'a>(call: &'a Expr, semantic: &'a SemanticModel) -> Option<
                 range: _,
             },
         range: _,
-    }) = &call
-    else {
-        return None;
-    };
+    } = call.as_call_expr()?;
     if args.len() != 2 {
         return None;
     }
     if !keywords.is_empty() {
         return None;
     }
-    let Expr::Name(ast::ExprName { id: func_name, .. }) = func.as_ref() else {
-        return None;
-    };
-    if func_name != "isinstance" {
-        return None;
-    }
-    if !semantic.is_builtin("isinstance") {
+    if !semantic.match_builtin_expr(func, "isinstance") {
         return None;
     }
 
@@ -394,7 +385,7 @@ pub(crate) fn duplicate_isinstance_call(checker: &mut Checker, expr: &Expr) {
                 },
                 expr.range(),
             );
-            if !contains_effect(target, |id| checker.semantic().is_builtin(id)) {
+            if !contains_effect(target, |id| checker.semantic().has_builtin_binding(id)) {
                 // Grab the types used in each duplicate `isinstance` call (e.g., `int` and `str`
                 // in `isinstance(obj, int) or isinstance(obj, str)`).
                 let types: Vec<&Expr> = indices
@@ -529,13 +520,17 @@ pub(crate) fn compare_with_tuple(checker: &mut Checker, expr: &Expr) {
         // Avoid rewriting (e.g.) `a == "foo" or a == f()`.
         if comparators
             .iter()
-            .any(|expr| contains_effect(expr, |id| checker.semantic().is_builtin(id)))
+            .any(|expr| contains_effect(expr, |id| checker.semantic().has_builtin_binding(id)))
         {
             continue;
         }
 
         // Avoid removing comments.
-        if checker.indexer().has_comments(expr, checker.locator()) {
+        if checker
+            .indexer()
+            .comment_ranges()
+            .has_comments(expr, checker.locator())
+        {
             continue;
         }
 
@@ -623,7 +618,7 @@ pub(crate) fn expr_and_not_expr(checker: &mut Checker, expr: &Expr) {
         return;
     }
 
-    if contains_effect(expr, |id| checker.semantic().is_builtin(id)) {
+    if contains_effect(expr, |id| checker.semantic().has_builtin_binding(id)) {
         return;
     }
 
@@ -680,7 +675,7 @@ pub(crate) fn expr_or_not_expr(checker: &mut Checker, expr: &Expr) {
         return;
     }
 
-    if contains_effect(expr, |id| checker.semantic().is_builtin(id)) {
+    if contains_effect(expr, |id| checker.semantic().has_builtin_binding(id)) {
         return;
     }
 
@@ -757,14 +752,15 @@ fn is_short_circuit(
 
     for (index, (value, next_value)) in values.iter().tuple_windows().enumerate() {
         // Keep track of the location of the furthest-right, truthy or falsey expression.
-        let value_truthiness = Truthiness::from_expr(value, |id| checker.semantic().is_builtin(id));
+        let value_truthiness =
+            Truthiness::from_expr(value, |id| checker.semantic().has_builtin_binding(id));
         let next_value_truthiness =
-            Truthiness::from_expr(next_value, |id| checker.semantic().is_builtin(id));
+            Truthiness::from_expr(next_value, |id| checker.semantic().has_builtin_binding(id));
 
         // Keep track of the location of the furthest-right, non-effectful expression.
         if value_truthiness.is_unknown()
             && (!checker.semantic().in_boolean_test()
-                || contains_effect(value, |id| checker.semantic().is_builtin(id)))
+                || contains_effect(value, |id| checker.semantic().has_builtin_binding(id)))
         {
             furthest = next_value;
             continue;
