@@ -174,13 +174,9 @@ impl<'a> StringLiteralDisplay<'a> {
                     display_kind,
                 }
             }
-            ast::Expr::Dict(ast::ExprDict {
-                keys,
-                values,
-                range,
-            }) => {
-                let mut narrowed_keys = Vec::with_capacity(values.len());
-                for key in keys {
+            ast::Expr::Dict(dict @ ast::ExprDict { items, range }) => {
+                let mut narrowed_keys = Vec::with_capacity(items.len());
+                for key in dict.iter_keys() {
                     if let Some(key) = key {
                         // This is somewhat unfortunate,
                         // *but* using a dict for __slots__ is very rare
@@ -193,12 +189,11 @@ impl<'a> StringLiteralDisplay<'a> {
                 // `__slots__ = {"foo": "bar", **other_dict}`
                 // If `None` wasn't present in the keys,
                 // the length of the keys should always equal the length of the values
-                assert_eq!(narrowed_keys.len(), values.len());
-                let display_kind = DisplayKind::Dict { values };
+                assert_eq!(narrowed_keys.len(), items.len());
                 Self {
                     elts: Cow::Owned(narrowed_keys),
                     range: *range,
-                    display_kind,
+                    display_kind: DisplayKind::Dict { items },
                 }
             }
             _ => return None,
@@ -206,7 +201,7 @@ impl<'a> StringLiteralDisplay<'a> {
         Some(result)
     }
 
-    fn generate_fix(&self, items: &[&str], checker: &Checker) -> Option<Fix> {
+    fn generate_fix(&self, elements: &[&str], checker: &Checker) -> Option<Fix> {
         let locator = checker.locator();
         let is_multiline = locator.contains_line_break(self.range());
         let sorted_source_code = match (&self.display_kind, is_multiline) {
@@ -215,6 +210,7 @@ impl<'a> StringLiteralDisplay<'a> {
                     self.range(),
                     *sequence_kind,
                     locator,
+                    elements,
                 )?;
                 assert_eq!(analyzed_sequence.len(), self.elts.len());
                 analyzed_sequence.into_sorted_source_code(SORTING_STYLE, locator, checker.stylist())
@@ -224,12 +220,12 @@ impl<'a> StringLiteralDisplay<'a> {
             (DisplayKind::Sequence(sequence_kind), false) => sort_single_line_elements_sequence(
                 *sequence_kind,
                 &self.elts,
-                items,
+                elements,
                 locator,
                 SORTING_STYLE,
             ),
-            (DisplayKind::Dict { values }, false) => {
-                sort_single_line_elements_dict(&self.elts, items, values, locator)
+            (DisplayKind::Dict { items }, false) => {
+                sort_single_line_elements_dict(&self.elts, elements, items, locator)
             }
         };
         Some(Fix::safe_edit(Edit::range_replacement(
@@ -245,7 +241,7 @@ impl<'a> StringLiteralDisplay<'a> {
 #[derive(Debug)]
 enum DisplayKind<'a> {
     Sequence(SequenceKind),
-    Dict { values: &'a [ast::Expr] },
+    Dict { items: &'a [ast::DictItem] },
 }
 
 /// A newtype that zips together three iterables:
@@ -262,14 +258,14 @@ enum DisplayKind<'a> {
 struct DictElements<'a>(Vec<(&'a &'a str, &'a ast::Expr, &'a ast::Expr)>);
 
 impl<'a> DictElements<'a> {
-    fn new(elements: &'a [&str], key_elts: &'a [ast::Expr], value_elts: &'a [ast::Expr]) -> Self {
+    fn new(elements: &'a [&str], key_elts: &'a [ast::Expr], items: &'a [ast::DictItem]) -> Self {
         assert_eq!(key_elts.len(), elements.len());
-        assert_eq!(elements.len(), value_elts.len());
+        assert_eq!(elements.len(), items.len());
         assert!(
             elements.len() >= 2,
             "A sequence with < 2 elements cannot be unsorted"
         );
-        Self(izip!(elements, key_elts, value_elts).collect())
+        Self(izip!(elements, key_elts, items.iter().map(|item| &item.value)).collect())
     }
 
     fn last_item_index(&self) -> usize {
@@ -294,13 +290,13 @@ impl<'a> DictElements<'a> {
 /// `sequence_sorting.rs` if any other modules need it,
 /// but stays here for now, since this is currently the
 /// only module that needs it
-fn sort_single_line_elements_dict(
-    key_elts: &[ast::Expr],
-    elements: &[&str],
-    value_elts: &[ast::Expr],
+fn sort_single_line_elements_dict<'a>(
+    key_elts: &'a [ast::Expr],
+    elements: &'a [&str],
+    original_items: &'a [ast::DictItem],
     locator: &Locator,
 ) -> String {
-    let element_trios = DictElements::new(elements, key_elts, value_elts);
+    let element_trios = DictElements::new(elements, key_elts, original_items);
     let last_item_index = element_trios.last_item_index();
     let mut result = String::from('{');
     // We grab the original source-code ranges using `locator.slice()`
