@@ -49,6 +49,7 @@ pub enum PythonVersion {
     Py310,
     Py311,
     Py312,
+    Py313,
 }
 
 impl From<PythonVersion> for Pep440Version {
@@ -72,6 +73,7 @@ impl PythonVersion {
             Self::Py310 => (3, 10),
             Self::Py311 => (3, 11),
             Self::Py312 => (3, 12),
+            Self::Py313 => (3, 13),
         }
     }
 
