@@ -1,36 +1,31 @@
 use anyhow::anyhow;
+use lsp_types::Url;
 use rustc_hash::FxHashMap;
-use std::{
-    collections::BTreeMap,
-    path::{Path, PathBuf},
-    sync::Arc,
-};
+use std::path::PathBuf;
+use std::{collections::BTreeMap, path::Path, sync::Arc};
 
 use crate::{
     edit::{DocumentKey, DocumentVersion, NotebookDocument},
     PositionEncoding, TextDocument,
 };
 
-use super::{
-    settings::{self, ResolvedClientSettings},
-    ClientSettings,
-};
+use super::{settings::ResolvedClientSettings, ClientSettings};
 
 mod ruff_settings;
 
 pub(crate) use ruff_settings::RuffSettings;
 
-type DocumentIndex = FxHashMap<PathBuf, DocumentController>;
-type NotebookCellIndex = FxHashMap<lsp_types::Url, PathBuf>;
 type SettingsIndex = BTreeMap<PathBuf, WorkspaceSettings>;
 
 /// Stores and tracks all open documents in a session, along with their associated settings.
 #[derive(Default)]
 pub(crate) struct Index {
-    /// Maps all document file paths to the associated document controller
-    documents: DocumentIndex,
-    /// Maps opaque cell URLs to a notebook path
-    notebook_cells: NotebookCellIndex,
+    /// Maps all document file URLs to the associated document controller
+    documents: FxHashMap<Url, DocumentController>,
+
+    /// Maps opaque cell URLs to a notebook URL (document)
+    notebook_cells: FxHashMap<Url, Url>,
+
     /// Maps a workspace folder root to its settings.
     settings: SettingsIndex,
 }
@@ -38,10 +33,11 @@ pub(crate) struct Index {
 /// Settings associated with a workspace.
 struct WorkspaceSettings {
     client_settings: ResolvedClientSettings,
-    workspace_settings_index: ruff_settings::RuffSettingsIndex,
+    ruff_settings: ruff_settings::RuffSettingsIndex,
 }
 
 /// A mutable handler to an underlying document.
+#[derive(Debug)]
 enum DocumentController {
     Text(Arc<TextDocument>),
     Notebook(Arc<NotebookDocument>),
@@ -53,14 +49,15 @@ enum DocumentController {
 #[derive(Clone)]
 pub(crate) enum DocumentQuery {
     Text {
-        file_path: PathBuf,
+        file_url: Url,
         document: Arc<TextDocument>,
         settings: Arc<RuffSettings>,
     },
     Notebook {
         /// The selected notebook cell, if it exists.
-        cell_uri: Option<lsp_types::Url>,
-        file_path: PathBuf,
+        cell_url: Option<Url>,
+        /// The URL of the notebook.
+        file_url: Url,
         notebook: Arc<NotebookDocument>,
         settings: Arc<RuffSettings>,
     },
@@ -68,42 +65,38 @@ pub(crate) enum DocumentQuery {
 
 impl Index {
     pub(super) fn new(
-        workspace_folders: Vec<(PathBuf, ClientSettings)>,
+        workspace_folders: Vec<(Url, ClientSettings)>,
         global_settings: &ClientSettings,
-    ) -> Self {
+    ) -> crate::Result<Self> {
         let mut settings_index = BTreeMap::new();
-        for (path, workspace_settings) in workspace_folders {
+        for (url, workspace_settings) in workspace_folders {
             Self::register_workspace_settings(
                 &mut settings_index,
-                path,
+                &url,
                 Some(workspace_settings),
                 global_settings,
-            );
+            )?;
         }
 
-        Self {
+        Ok(Self {
             documents: FxHashMap::default(),
             notebook_cells: FxHashMap::default(),
             settings: settings_index,
-        }
+        })
     }
 
-    pub(super) fn text_document_urls(&self) -> impl Iterator<Item = lsp_types::Url> + '_ {
+    pub(super) fn text_document_urls(&self) -> impl Iterator<Item = &Url> + '_ {
         self.documents
             .iter()
             .filter(|(_, doc)| doc.as_text().is_some())
-            .map(|(path, _)| {
-                lsp_types::Url::from_file_path(path).expect("valid file path should convert to URL")
-            })
+            .map(|(url, _)| url)
     }
 
-    pub(super) fn notebook_document_urls(&self) -> impl Iterator<Item = lsp_types::Url> + '_ {
+    pub(super) fn notebook_document_urls(&self) -> impl Iterator<Item = &Url> + '_ {
         self.documents
             .iter()
             .filter(|(_, doc)| doc.as_notebook().is_some())
-            .map(|(path, _)| {
-                lsp_types::Url::from_file_path(path).expect("valid file path should convert to URL")
-            })
+            .map(|(url, _)| url)
     }
 
     pub(super) fn update_text_document(
@@ -128,22 +121,17 @@ impl Index {
         Ok(())
     }
 
-    pub(super) fn key_from_url(&self, url: &lsp_types::Url) -> Option<DocumentKey> {
-        if self.notebook_cells.contains_key(url) {
-            return Some(DocumentKey::NotebookCell(url.clone()));
+    pub(super) fn key_from_url(&self, url: Url) -> DocumentKey {
+        if self.notebook_cells.contains_key(&url) {
+            DocumentKey::NotebookCell(url)
+        } else if Path::new(url.path())
+            .extension()
+            .map_or(false, |ext| ext.eq_ignore_ascii_case("ipynb"))
+        {
+            DocumentKey::Notebook(url)
+        } else {
+            DocumentKey::Text(url)
         }
-        let path = url.to_file_path().ok()?;
-        Some(
-            match path
-                .extension()
-                .unwrap_or_default()
-                .to_str()
-                .unwrap_or_default()
-            {
-                "ipynb" => DocumentKey::Notebook(path),
-                _ => DocumentKey::Text(path),
-            },
-        )
     }
 
     pub(super) fn update_notebook_document(
@@ -161,7 +149,7 @@ impl Index {
             ..
         }) = cells.as_ref().and_then(|cells| cells.structure.as_ref())
         {
-            let Some(path) = self.path_for_key(key).cloned() else {
+            let Some(path) = self.url_for_key(key).cloned() else {
                 anyhow::bail!("Tried to open unavailable document `{key}`");
             };
 
@@ -190,24 +178,29 @@ impl Index {
 
     pub(super) fn open_workspace_folder(
         &mut self,
-        path: PathBuf,
+        url: &Url,
         global_settings: &ClientSettings,
-    ) {
+    ) -> crate::Result<()> {
         // TODO(jane): Find a way for workspace client settings to be added or changed dynamically.
-        Self::register_workspace_settings(&mut self.settings, path, None, global_settings);
+        Self::register_workspace_settings(&mut self.settings, url, None, global_settings)
     }
 
     fn register_workspace_settings(
         settings_index: &mut SettingsIndex,
-        workspace_path: PathBuf,
+        workspace_url: &Url,
         workspace_settings: Option<ClientSettings>,
         global_settings: &ClientSettings,
-    ) {
+    ) -> crate::Result<()> {
         let client_settings = if let Some(workspace_settings) = workspace_settings {
             ResolvedClientSettings::with_workspace(&workspace_settings, global_settings)
         } else {
             ResolvedClientSettings::global(global_settings)
         };
+
+        let workspace_path = workspace_url
+            .to_file_path()
+            .map_err(|()| anyhow!("workspace URL was not a file path!"))?;
+
         let workspace_settings_index = ruff_settings::RuffSettingsIndex::new(
             &workspace_path,
             client_settings.editor_settings(),
@@ -217,23 +210,31 @@ impl Index {
             workspace_path,
             WorkspaceSettings {
                 client_settings,
-                workspace_settings_index,
+                ruff_settings: workspace_settings_index,
             },
         );
+
+        Ok(())
     }
 
-    pub(super) fn close_workspace_folder(&mut self, workspace_path: &PathBuf) -> crate::Result<()> {
-        self.settings.remove(workspace_path).ok_or_else(|| {
+    pub(super) fn close_workspace_folder(&mut self, workspace_url: &Url) -> crate::Result<()> {
+        let workspace_path = workspace_url
+            .to_file_path()
+            .map_err(|()| anyhow!("workspace URL was not a file path!"))?;
+
+        self.settings.remove(&workspace_path).ok_or_else(|| {
             anyhow!(
-                "Tried to remove non-existent folder {}",
-                workspace_path.display()
+                "Tried to remove non-existent workspace URI {}",
+                workspace_url
             )
         })?;
+
         // O(n) complexity, which isn't ideal... but this is an uncommon operation.
         self.documents
-            .retain(|path, _| !path.starts_with(workspace_path));
+            .retain(|url, _| !Path::new(url.path()).starts_with(&workspace_path));
         self.notebook_cells
-            .retain(|_, path| !path.starts_with(workspace_path));
+            .retain(|_, url| !Path::new(url.path()).starts_with(&workspace_path));
+
         Ok(())
     }
 
@@ -242,70 +243,99 @@ impl Index {
         key: DocumentKey,
         global_settings: &ClientSettings,
     ) -> Option<DocumentQuery> {
-        let path = self.path_for_key(&key)?.clone();
+        let url = self.url_for_key(&key)?.clone();
+
         let document_settings = self
-            .settings_for_path(&path)
-            .map(|settings| settings.workspace_settings_index.get(&path))
+            .settings_for_url(&url)
+            .map(|settings| {
+                if let Ok(file_path) = url.to_file_path() {
+                    settings.ruff_settings.get(&file_path)
+                } else {
+                    // For a new unsaved and untitled document, use the ruff settings from the top of the workspace
+                    // but only IF:
+                    // * It is the only workspace
+                    // * The ruff setting is at the top of the workspace (in the root folder)
+                    // Otherwise, use the fallback settings.
+                    if self.settings.len() == 1 {
+                        let workspace_path = self.settings.keys().next().unwrap();
+                        settings.ruff_settings.get(&workspace_path.join("untitled"))
+                    } else {
+                        tracing::debug!("Use the fallback settings for the new document '{url}'.");
+                        settings.ruff_settings.fallback()
+                    }
+                }
+            })
             .unwrap_or_else(|| {
                 tracing::warn!(
                     "No settings available for {} - falling back to default settings",
-                    path.display()
+                    url
                 );
                 let resolved_global = ResolvedClientSettings::global(global_settings);
-                let root = path.parent().unwrap_or(&path);
+                // The path here is only for completeness, it's okay to use a non-existing path
+                // in case this is an unsaved (untitled) document.
+                let path = Path::new(url.path());
+                let root = path.parent().unwrap_or(path);
                 Arc::new(RuffSettings::fallback(
                     resolved_global.editor_settings(),
                     root,
                 ))
             });
 
-        let controller = self.documents.get(&path)?;
-        let cell_uri = match key {
-            DocumentKey::NotebookCell(uri) => Some(uri),
+        let controller = self.documents.get(&url)?;
+        let cell_url = match key {
+            DocumentKey::NotebookCell(cell_url) => Some(cell_url),
             _ => None,
         };
-        Some(controller.make_ref(cell_uri, path, document_settings))
+        Some(controller.make_ref(cell_url, url, document_settings))
     }
 
     /// Reloads relevant existing settings files based on a changed settings file path.
     /// This does not currently register new settings files.
-    pub(super) fn reload_settings(&mut self, changed_path: &PathBuf) {
-        let search_path = changed_path.parent().unwrap_or(changed_path);
-        for (root, settings) in self
-            .settings
-            .iter_mut()
-            .filter(|(path, _)| path.starts_with(search_path))
-        {
-            settings.workspace_settings_index = ruff_settings::RuffSettingsIndex::new(
+    pub(super) fn reload_settings(&mut self, changed_url: &Url) {
+        let Ok(changed_path) = changed_url.to_file_path() else {
+            // Files that don't map to a path can't be a workspace configuration file.
+            return;
+        };
+
+        let Some(enclosing_folder) = changed_path.parent() else {
+            return;
+        };
+
+        // TODO: I think this does not correctly reload settings when using `extend` and the extended
+        //  setting isn't in a parent folder.
+        for (root, settings) in self.settings.range_mut(enclosing_folder.to_path_buf()..) {
+            if !root.starts_with(enclosing_folder) {
+                break;
+            }
+
+            settings.ruff_settings = ruff_settings::RuffSettingsIndex::new(
                 root,
                 settings.client_settings.editor_settings(),
             );
         }
     }
 
-    pub(super) fn open_text_document(&mut self, path: PathBuf, document: TextDocument) {
+    pub(super) fn open_text_document(&mut self, url: Url, document: TextDocument) {
         self.documents
-            .insert(path, DocumentController::new_text(document));
+            .insert(url, DocumentController::new_text(document));
     }
 
-    pub(super) fn open_notebook_document(&mut self, path: PathBuf, document: NotebookDocument) {
-        for url in document.urls() {
-            self.notebook_cells.insert(url.clone(), path.clone());
+    pub(super) fn open_notebook_document(&mut self, notebook_url: Url, document: NotebookDocument) {
+        for cell_url in document.urls() {
+            self.notebook_cells
+                .insert(cell_url.clone(), notebook_url.clone());
         }
         self.documents
-            .insert(path, DocumentController::new_notebook(document));
+            .insert(notebook_url, DocumentController::new_notebook(document));
     }
 
     pub(super) fn close_document(&mut self, key: &DocumentKey) -> crate::Result<()> {
-        let Some(path) = self.path_for_key(key).cloned() else {
-            anyhow::bail!("Tried to open unavailable document `{key}`");
+        let Some(url) = self.url_for_key(key).cloned() else {
+            anyhow::bail!("Tried to close unavailable document `{key}`");
         };
 
-        let Some(controller) = self.documents.remove(&path) else {
-            anyhow::bail!(
-                "tried to close document that didn't exist at {}",
-                path.display()
-            )
+        let Some(controller) = self.documents.remove(&url) else {
+            anyhow::bail!("tried to close document that didn't exist at {}", url)
         };
         if let Some(notebook) = controller.as_notebook() {
             for url in notebook.urls() {
@@ -321,13 +351,13 @@ impl Index {
         &self,
         key: &DocumentKey,
         global_settings: &ClientSettings,
-    ) -> settings::ResolvedClientSettings {
-        let Some(path) = self.path_for_key(key) else {
+    ) -> ResolvedClientSettings {
+        let Some(url) = self.url_for_key(key) else {
             return ResolvedClientSettings::global(global_settings);
         };
         let Some(WorkspaceSettings {
             client_settings, ..
-        }) = self.settings_for_path(path)
+        }) = self.settings_for_url(url)
         else {
             return ResolvedClientSettings::global(global_settings);
         };
@@ -338,22 +368,38 @@ impl Index {
         &mut self,
         key: &DocumentKey,
     ) -> crate::Result<&mut DocumentController> {
-        let Some(path) = self.path_for_key(key).cloned() else {
+        let Some(url) = self.url_for_key(key).cloned() else {
             anyhow::bail!("Tried to open unavailable document `{key}`");
         };
-        let Some(controller) = self.documents.get_mut(&path) else {
-            anyhow::bail!("Document controller not available at `{}`", path.display());
+        let Some(controller) = self.documents.get_mut(&url) else {
+            anyhow::bail!("Document controller not available at `{}`", url);
         };
         Ok(controller)
     }
 
-    fn path_for_key<'a>(&'a self, key: &'a DocumentKey) -> Option<&'a PathBuf> {
+    fn url_for_key<'a>(&'a self, key: &'a DocumentKey) -> Option<&'a Url> {
         match key {
             DocumentKey::Notebook(path) | DocumentKey::Text(path) => Some(path),
             DocumentKey::NotebookCell(uri) => self.notebook_cells.get(uri),
         }
     }
 
+    fn settings_for_url(&self, url: &Url) -> Option<&WorkspaceSettings> {
+        if let Ok(path) = url.to_file_path() {
+            self.settings_for_path(&path)
+        } else {
+            // If there's only a single workspace, use that configuration for an untitled document.
+            if self.settings.len() == 1 {
+                tracing::debug!(
+                    "Falling back to configuration of the only active workspace for the new document '{url}'."
+                );
+                self.settings.values().next()
+            } else {
+                None
+            }
+        }
+    }
+
     fn settings_for_path(&self, path: &Path) -> Option<&WorkspaceSettings> {
         self.settings
             .range(..path.to_path_buf())
@@ -373,19 +419,19 @@ impl DocumentController {
 
     fn make_ref(
         &self,
-        cell_uri: Option<lsp_types::Url>,
-        file_path: PathBuf,
+        cell_url: Option<Url>,
+        file_url: Url,
         settings: Arc<RuffSettings>,
     ) -> DocumentQuery {
         match &self {
             Self::Notebook(notebook) => DocumentQuery::Notebook {
-                cell_uri,
-                file_path,
+                cell_url,
+                file_url,
                 notebook: notebook.clone(),
                 settings,
             },
             Self::Text(document) => DocumentQuery::Text {
-                file_path,
+                file_url,
                 document: document.clone(),
                 settings,
             },
@@ -426,12 +472,12 @@ impl DocumentQuery {
     /// Retrieve the original key that describes this document query.
     pub(crate) fn make_key(&self) -> DocumentKey {
         match self {
-            Self::Text { file_path, .. } => DocumentKey::Text(file_path.clone()),
+            Self::Text { file_url, .. } => DocumentKey::Text(file_url.clone()),
             Self::Notebook {
-                cell_uri: Some(cell_uri),
+                cell_url: Some(cell_uri),
                 ..
             } => DocumentKey::NotebookCell(cell_uri.clone()),
-            Self::Notebook { file_path, .. } => DocumentKey::Notebook(file_path.clone()),
+            Self::Notebook { file_url, .. } => DocumentKey::Notebook(file_url.clone()),
         }
     }
 
@@ -465,7 +511,7 @@ impl DocumentQuery {
     /// Get the source type of the document associated with this query.
     pub(crate) fn source_type(&self) -> ruff_python_ast::PySourceType {
         match self {
-            Self::Text { .. } => ruff_python_ast::PySourceType::Python,
+            Self::Text { .. } => ruff_python_ast::PySourceType::from(self.virtual_file_path()),
             Self::Notebook { .. } => ruff_python_ast::PySourceType::Ipynb,
         }
     }
@@ -478,20 +524,39 @@ impl DocumentQuery {
         }
     }
 
-    /// Get the underlying file path for the document selected by this query.
-    pub(crate) fn file_path(&self) -> &PathBuf {
+    /// Get the URL for the document selected by this query.
+    pub(crate) fn file_url(&self) -> &Url {
         match self {
-            Self::Text { file_path, .. } | Self::Notebook { file_path, .. } => file_path,
+            Self::Text { file_url, .. } | Self::Notebook { file_url, .. } => file_url,
         }
     }
 
+    /// Get the path for the document selected by this query.
+    ///
+    /// Returns `None` if this is an unsaved (untitled) document.
+    ///
+    /// The path isn't guaranteed to point to a real path on the filesystem. This is the case
+    /// for unsaved (untitled) documents.
+    pub(crate) fn file_path(&self) -> Option<PathBuf> {
+        self.file_url().to_file_path().ok()
+    }
+
+    /// Get the path for the document selected by this query, ignoring whether the file exists on disk.
+    ///
+    /// Returns the URL's path if this is an unsaved (untitled) document.
+    pub(crate) fn virtual_file_path(&self) -> &Path {
+        Path::new(self.file_url().path())
+    }
+
     /// Attempt to access the single inner text document selected by the query.
     /// If this query is selecting an entire notebook document, this will return `None`.
     pub(crate) fn as_single_document(&self) -> Option<&TextDocument> {
         match self {
             Self::Text { document, .. } => Some(document),
             Self::Notebook {
-                notebook, cell_uri, ..
+                notebook,
+                cell_url: cell_uri,
+                ..
             } => cell_uri
                 .as_ref()
                 .and_then(|cell_uri| notebook.cell_document_by_uri(cell_uri)),
